/******************************************************************************
     * File: Protocol.h
     * Description: Класс протокола передачи данных
     * Created: 22 июня 2015
     * Last Modifiede: 09 августа 2015  
******************************************************************************/

/******************************************************************************
     * Based on: Protocol.h - https://code.google.com/p/multiwii/
******************************************************************************/

/******************************************************************************
     * CodeStyle: 3  
******************************************************************************/

#ifndef PROTOCOL_H_
#define PROTOCOL_H_

//********************  Объявление переменных, констант  ********************//
// Структура хранящая текущие значения нажатых кнопок и положения стиков джойстика от PS2.
extern ps2Struct ps2S;

// Структура хранящая предыдущие значения нажатых кнопок и положения стиков джойстика от PS2.
extern rovDataStruct rovDataS;
//********************  /Объявление переменных, констант  *******************//

//**********************  Объявление приватные функции  *********************//
/// <summary>
/// Сохраняем отправляемый 8 бит в буфер TX и считаем контрольную сумму.
/// </summary>
static void Serialize8(uint8_t a);

/// <summary>
/// Сохраняем отправляемый 16 бит в буфер TX и считаем контрольную сумму.
/// </summary>
static void Serialize16(uint16_t a);

/// <summary>
/// Сохраняем отправляемый 32 бит в буфер TX и считаем контрольную сумму.
/// </summary>
static void Serialize32(uint32_t a);

/// <summary>
/// Читаем 8 бит данных из буфера.
/// </summary>
static uint8_t Read8();

/// <summary>
/// Читаем 16 бит данных из буфера.
/// </summary>
static uint16_t Read16();

/// <summary>
/// Читаем 32 бит данных из буфера.
/// </summary>
static uint32_t Read32();

/// <summary>
/// Читаем структуру из буфера.
/// </summary>
static void ReadStruct(uint8_t *cb,uint8_t siz);

/// <summary>
/// Конвертируем байты в структуру.
/// </summary>
static void ConversionBytesIntoStructure();
//**********************  /Объявление приватные функции *********************//

//**********************  Объявление публичных функции  *********************//
/// <summary>
/// Сохраняем структуру в буфер TX и считаем контрольную сумму.
/// </summary>
void SerializeStruct(uint8_t *cb,uint8_t siz);

/// <summary>
/// Сохраняем заголовок команды в буфер TX и считаем контрольную сумму.
/// </summary>
void HeadCmdSend(uint8_t err, uint8_t s, uint8_t c);

/// <summary>
/// Сохраняем хвост команды в буфер TX, считаем контрольную сумму и отправляем данные.
/// </summary>
void TailCmdSend(void);

/// <summary>
/// Получение данных.
/// </summary>
void UARTGetData(void);

#ifdef UART_HARDWARE_DEBUG
  void WriteDebug(uint8_t c);

  void WriteDebug(const uint8_t *buffer, size_t size);
  
  inline void WriteDebug(unsigned long n);
  
  inline void WriteDebug(long n);
  
  inline void WriteDebug(unsigned int n);
  
  inline void WriteDebug(int n);
  
  void WriteDebug(const char *buffer, size_t size);
  
  void WriteDebug(const char *str);
  
  void PrintDebugNumber(unsigned long n, uint8_t base);
  
  void PrintDebugFloat(double number, uint8_t digits);
  
  void PrintDebug(const char str[]);
  
  void PrintDebug(char c);
  
  void PrintDebug(unsigned char b, uint8_t base);
  
  void PrintDebug(int n, uint8_t base);
  
  void PrintDebug(unsigned int n, uint8_t base);
  
  void PrintDebug(long n, uint8_t base);
  
  void PrintDebug(unsigned long n, uint8_t base);
  
  void PrintDebug(double n, uint8_t digits);
  
  void PrintDebug(const String &s);
  
  void PrintDebug(unsigned char n);
  
  void PrintDebug(int n);
  
  void PrintDebug(unsigned int n);
  
  void PrintDebug(long n);
  
  void PrintDebug(unsigned long n);
  
  void PrintDebug(double n);
#endif
//********************** /Объявление публичных функции  *********************//

#endif /* PROTOCOL_H_ */

