#ifndef UART_H_
#define UART_H_

//********************  Объявление переменных, констант  ********************//

// Определяем кол-во UART-тов в зависимости от платы
#if defined(ARDUINO_MEGA_2560)
  // Если ArduinoMega
  #define UARTNumber 4
#elif defined(ARDUINO_PRO_MICRO)
  // Если ArduinoProMicro
  #define UARTNumber 2
#else
  // Иначе предполагается ArduinoProMini
  #define UARTNumber 1
#endif

// Размер буфера RX
#define RXBufferSize 64

// Размер буфера TX
#define TXBufferSize 64

//********************  /Объявление переменных, констант  *******************//

//**********************  Объявление публичных функций  *********************//
/// <summary>
/// Инициализируем и открываем UART порт 
/// </summary>
/// <param name="port">Номер порта</param>
/// <param name="baud">Скорость работы порта</param>
void UARTOpen(uint8_t port, uint32_t baud);

/// <summary>
/// Закрываем UART порт
/// </summary> 
/// <param name="port">Номер порта</param>
void UARTClose(uint8_t port);

/// <summary>
/// Получает количество байт доступных для чтения из буфера RX
/// </summary> 
/// <param name="port">Номер порта</param>
uint8_t UARTAvailable(uint8_t port);

/// <summary>
/// Читаем данные из буфера RX
/// </summary> 
/// <param name="port">Номер порта</param>
uint8_t UARTRead(uint8_t port);

/// <summary>
/// Сохраняем отправляемый байт в буфер TX
/// </summary>
/// <param name="port">Номер порта</param>
/// <param name="a">Отправляемый байт</param>
void UARTSerialize(uint8_t port,uint8_t a);

/// <summary>
/// Получает количество байт доступных для отправки в буфере RX
/// </summary>
/// <param name="port">Номер порта</param>
uint8_t UARTUsedTXBuff(uint8_t port);

/// <summary>
/// Отправляем данные по UART, включаем прерывание по опустошению буфера UART
/// </summary> 
/// <param name="port">Номер порта</param>
void UARTSendData(uint8_t port);

/// <summary>
/// Сохраняем отправляемый байт в буфер TX и отправляем данные по UART, включаем прерывание по опустошению буфера UART
/// </summary> 
/// <param name="port">Номер порта</param>
/// <param name="c">Отправляемый байт</param>
void UARTWrite(uint8_t port,uint8_t c);

//*********************  /Объявление публичных функций  *********************//

//**********************  Объявление приватных функций  *********************//
static inline void StoreUARTInBuf(uint8_t data, uint8_t portnum) __attribute__((always_inline));
//*********************  /Объявление приватных функций  *********************//

#endif /* UART_H_ */

