//*******************************  Библиотеки  ******************************//
#include "Arduino.h"

#include "Config.h"

#include "Type.h"

#include "FastIO.h"

#include "UART.h"

#include "Protocol.h"

#ifdef UART_SOFTWARE_DEBUG 
  #include <SoftwareSerial.h> 
#endif
//*******************************  /Библиотеки  *****************************//

//********************** Объявление переменных, констант ********************//
#ifdef UART_SOFTWARE_DEBUG
  SoftwareSerial UARTSoftwareDebug(10, 11); 
#endif    

// Переменная для хранения предыдущего значения micros(), используется для ответа по RS485
uint32_t answerRS485PreviousT = 0;
  
  // Переменная хранящая период времени, после которого можно посылать ответ по RS485.
uint32_t answerRS485T = 500;

// Переменная хранящая начало цикла.
uint32_t timeCycleBegin = 0;
  
// Переменная хранящая время цикла.
uint32_t timeCycle = 0;
  
// Переменная хранящая максимальное время цикла.
uint32_t timeCycleMax = 0;
//********************** /Объявление переменных, констант ********************//

//********** Функции работы с таймерами на основе millis(), micros() ********//
/// <summary>
/// Переменная хранящее максимальное значение unsigned long
/// </summary>
#define unsignedLongMax 4294967295UL

/// <summary>
/// Переменная хранящее 1, для проверки скорости выполнения цикла
/// </summary>
#define digitalOne 1UL

/// <summary>
/// Получение разницы между 2 uint32_t
/// </summary>
uint32_t GetDifferenceULong(uint32_t BeginTime, uint32_t EndTime)
{
  if (EndTime < BeginTime)
  {
    // Защита от переполнения
    return unsignedLongMax - BeginTime + EndTime + digitalOne;
  }
  else
  {
    return EndTime - BeginTime;
  }
}
//********* /Функции работы с таймерами на основе millis(), micros() ********//

//************************** Функции ответа Rov по UART **********************//
void rovSendAnswer(void)
{     
  HeadCmdSend(0, 1, 2);
        
  rovDataS.errore = 1;
                  
  SerializeStruct((uint8_t*)&rovDataS, 1);

  TailCmdSend();
}
//************************** /Функции ответа Rov по UART *********************//

//****************************** Основные функции ****************************//
/// <summary>
/// Настройка при запуске arduino
/// </summary>
void setup()
{
  // Инициализация пинов.
  MC_SET_PIN_OUTPUT(RS485_PIN_MASTER);
  MC_SET_PIN_OUTPUT(LED_PIN);
  
  // Инициализируем и открываем UART порт связи с Rov
  UARTOpen(UART_PORT, UART_PORT_SPEED);
  
  // Инициализация Software UART.
  #ifdef UART_SOFTWARE_DEBUG 
    UARTSoftwareDebug.begin(UART_DEBUG_SPEED);
  #endif 
}

/// <summary>
/// Основной цикл.
/// </summary>
void loop()
{  
  // Проверка скорости цикла.
  #if defined(DEBUG_SPEED_CYCLE)
    timeCycleBegin = micros();
  #endif
  
  // Пришли данные с джойстика.
  if (ps2S.statuswork == 1)
  {   
    #ifdef UART_USE_RS485
      if (answerRS485PreviousT == 0)
      {
        answerRS485PreviousT = micros();
      }
    #endif       
                 
    ps2S.statuswork = 2;  
    ps2S.bfirst = ps2S.bsecond = 0;
  }

  // Посылаем ответ.
  #if defined(UART_USE_RS485)
    if (answerRS485PreviousT && GetDifferenceULong(answerRS485PreviousT, micros()) > answerRS485T)
    {
      rovSendAnswer();
        
      answerRS485PreviousT = 0;
    }
  #endif  
  
  // Получение данных.
  UARTGetData();   
 
  // Проверяем скорость цикла.
  #if defined(DEBUG_SPEED_CYCLE) && defined(UART_SOFTWARE_DEBUG)
    timeCycle = GetDifferenceULong(timeCycleBegin, micros());
      
    if (timeCycle > timeCycleMax)
    {
      timeCycleMax = timeCycle;
      
      UARTSoftwareDebug.print("Time Cycle Max - ");  
      UARTSoftwareDebug.println(timeCycleMax);
    }
  #endif 
}
//****************************** /Основные функции ***************************//


